////////////////////////////////////////////////////////////////////////////////
//
// Time-stamp: "30.12.12 19:37 fixpoint.h.tmp klaus?wachtler.de"
//
////////////////////////////////////////////////////////////////////////////////
//
//
//     C++ templates defining numerical fixed point types
//     --------------------------------------------------
//
//
// The functions are intended to be light weight, fast and thus
// useful on small systems like micro controllers without hardware
// floating point unit.
//
// Speed and code size might be a little worse than integral
// calculations but much better than floating point arithmetic
// especially on systems without FPU.
//
// Tested with g++ 4.3.2 and avr-gcc 4.3.2.
//
// ATTENTION!
// The code is tested only partially (as from 09.07.2009).
// Not usable for production code.
//
////////////////////////////////////////////////////////////////////////////////
//
// (C) 2009-2011
//     Klaus Wachtler
//     Breidingstr. 17
//     D-29614 Soltau
//     Phone: +49-171-4553039
//     Mail:  fixpoint<you guess the character missing here?>wachtler.de
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 3 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, see <http://www.gnu.org/licenses/>.
//
////////////////////////////////////////////////////////////////////////////////
//
// Template arguments:
//////////////////////
//
// - TBASE is some usual signed or unsigned integral type like
//   signed/unsigned int, int32_t, uint32_t etc..
//   TBASE is used to keep an internal value which has to be scaled to
//   get the correct number.
//
// - TTEMP is another integral type used for values subject to
//   overflow if TBASE would be used.
//   You might want to use the same type for TBASE and TTEMP (like
//   fixpoint< int16_t, int16_t >) as long as you ensure to keep
//   intermediate results like in a*b within valid range.
//   If in doubt choose TTEMP one step larger than TBASE, so
//   fixpoint< uint8_t, uint16_t > or fixpoint< int32_t, int64_t >
//   seem to be a good choice.
//   Default ist TBASE.
//
// - LFRACTIONAL scales this value and determines range and
//   accuracy. It counts in binary digits.
//   A positive value of LFRACTIONAL gives a type with reduced range
//   and accuracy better than 1 by using LFRACTIONAL binary digits as
//   fractional part (LFRACTIONAL digits are used for the fractional
//   part, the rest of TBASE is used for the integer part),
//   while negative values of LFRACTIONAL give an extended range with
//   less accuracy (compared to TBASE).
//   * Example 1:
//     fixpoint< uint16_t, uint32_t, 3 > is based on uint16_t scaled
//     by 2**3=8.
//     While uint16_t has a range from 0 to 65535 with steps of 1
//     fixpoint< uint16_t, uint32_t, 33 > has values from 0 to
//     8191.875 (65535/8) with steps of 1/8=0.125. The fractional part
//     is 3 binary digits.
//     Possible values are 0, 0.125, 0.25, 0.375, 0.5 ... 8191.875.
//   * Example 2:
//     fixpoint< uint8_t, uint16_t, -2 > is based on uint8_t scaled
//     by 2**(-2)=1/4.
//     The values range from 0 (0*4) to 1020 (255*4) in steps of 4.
//     Possible values are 0, 4, 8, 12, ... 1020.
//   * Example 3:
//     fixpoint< uint8_t, uint16_t, 0 > is based on uint8_t without scaling.
//     This resembles uint8_t with values from 0 to 255.
//   * Example 4:
//     fixpoint< int8_t, int16_t, -2 > is based on int8_t scaled
//     by 2**(-2)=1/4.
//     The values range from -512 (-128*4) to +508 (127*4) in steps of 4.
//     Possible values are -512, -508, -502, ... -4, 0, +4, ... +508.
//   Default value for LFRACTIONAL is half of the bit count of TBASE.
//   E.g. fixpoint< uint16_t > has 16 binary digits with 8 of them in
//   the fractional part (as fixpoint< uint16_t, uint16_t, 8 >).
//
// - ROUNDTONEAREST determines if assignments and initializations
//   cut off trailing digits (false) or rounds up to the nearest
//   integral value (true). This will be more accurate, but a little
//   slower in some (rare) cases.
//   Default is true.
//
////////////////////////////////////////////////////////////////////////////////
//
// Usage:
/////////
//
// Compilation:
// ------------
//
// - Plain C is not supported. Use C++.
// - Use #include <fixpoint.h> and give -I... if necessary.
// - No libs needed (everything is inlined).
// - Compile with -DNO_STDSTRING or #define NO_STDSTRING before
//   #include <fixpoint.h> if no std::string is available.
// - Compile with -DNO_STDIOSTREAM or #define NO_STDIOSTREAM before
//   #include <fixpoint.h> if no iostreams are available.
// - Compile with -DDEBUG_FIXPOINT_... for special debug features, see
//   ``Debug options'' below.
// - Don't worry about warnings saying "shift count is negative".
//   This occurs in dead code which will be optimized away by the
//   compiler later.
// - The code will benefit extremely from compiler optimisation.
//   Do not forget to use -O2, -O3 or -Os at least as soon as you need
//   no debugging.
//
//
// Object creation:
// ----------------
//
// fixpoint objects can be created either:
// - without arguments (resulting in a 0 value)
// - an integral value (with their base type); values being not
//   integral can not be defined this way even if LFRACTIONAL is
//   positive
// - two integral values a and b (resulting in a/b, which will be cut
//   off or rounded to a valid value  depending on template argument
//   ROUNDTONEAREST); this is done by a template method with any
//   integral types for a and b and usual C style promotions
//   performing the division.
//   If LFRACTIONAL is positive the type TTEMP should be wide enough to
//   hold a<<LFRACTIONAL. If LFRACTIONAL is negative TTEMP should be
//   able to keep b<<LFRACTIONAL.
// - a floating point number (which will be cut off or rounded
//   to a valid value depending on template argument ROUNDTONEAREST)
//
//
// Valid operations:
// -----------------
//
//            | type of a | type of b | result   | lvalue | notes
// -----------+-----------+-----------+----------+--------+------------------------------
// f()        |           |           | fixpoint |        | constructor
// -----------+-----------+-----------+----------+--------+------------------------------
// f(a)       | base type |           | fixpoint |        | constructor (1)
// -----------+-----------+-----------+----------+--------+------------------------------
// f(a)       | double    |           | fixpoint |        | constructor
// -----------+-----------+-----------+----------+--------+------------------------------
// f(a,b)     | integral  | integral  | fixpoint |        | constructor (1)
// -----------+-----------+-----------+----------+--------+------------------------------
// !a         | fixpoint  |           | bool     |        | true if a==0
// -----------+-----------+-----------+----------+--------+------------------------------
// ++a        | fixpoint  |           | fixpoint | yes    | nop if LFRACTIONAL<0
// -----------+-----------+-----------+----------+--------+------------------------------
// a++        | fixpoint  |           | fixpoint |        | nop if LFRACTIONAL<0
// -----------+-----------+-----------+----------+--------+------------------------------
// --a        | fixpoint  |           | fixpoint | yes    | nop if LFRACTIONAL<0
// -----------+-----------+-----------+----------+--------+------------------------------
// a--        | fixpoint  |           | fixpoint |        | nop if LFRACTIONAL<0
// -----------+-----------+-----------+----------+--------+------------------------------
// +a         | fixpoint  |           | fixpoint |        | nop
// -----------+-----------+-----------+----------+--------+------------------------------
// -a         | fixpoint  |           | fixpoint |        |
// -----------+-----------+-----------+----------+--------+------------------------------
// a*b        | fixpoint  | fixpoint  | fixpoint |        | (1)
// -----------+-----------+-----------+----------+--------+------------------------------
// a/b        | fixpoint  | fixpoint  | fixpoint |        |
// -----------+-----------+-----------+----------+--------+------------------------------
// a%b        | fixpoint  | fixpoint  | fixpoint |        | x-n*y with n=x/y rounded -> 0
// -----------+-----------+-----------+----------+--------+------------------------------
// a+b        | fixpoint  | fixpoint  | fixpoint |        |
// -----------+-----------+-----------+----------+--------+------------------------------
// a-b        | fixpoint  | fixpoint  | fixpoint |        |
// -----------+-----------+-----------+----------+--------+------------------------------
// a<b        | fixpoint  | fixpoint  | bool     |        |
// -----------+-----------+-----------+----------+--------+------------------------------
// a>b        | fixpoint  | fixpoint  | bool     |        |
// -----------+-----------+-----------+----------+--------+------------------------------
// a<=b       | fixpoint  | fixpoint  | bool     |        |
// -----------+-----------+-----------+----------+--------+------------------------------
// a>=b       | fixpoint  | fixpoint  | bool     |        |
// -----------+-----------+-----------+----------+--------+------------------------------
// ==         | fixpoint  | fixpoint  | bool     |        |
// -----------+-----------+-----------+----------+--------+------------------------------
// !=         | fixpoint  | fixpoint  | bool     |        |
// -----------+-----------+-----------+----------+--------+------------------------------
// a&&b       | fixpoint  | fixpoint  | bool     |        |
// -----------+-----------+-----------+----------+--------+------------------------------
// a||b       | fixpoint  | fixpoint  | bool     |        |
// -----------+-----------+-----------+----------+--------+------------------------------
// a=b        | fixpoint  | any (2)   | fixpoint |        |
// -----------+-----------+-----------+----------+--------+------------------------------
// a+=b       | fixpoint  | any (2)   | fixpoint |        |
// -----------+-----------+-----------+----------+--------+------------------------------
// a-=b       | fixpoint  | any (2)   | fixpoint |        |
// -----------+-----------+-----------+----------+--------+------------------------------
// a*=b       | fixpoint  | any (2)   | fixpoint |        | (1)
// -----------+-----------+-----------+----------+--------+------------------------------
// a/=b       | fixpoint  | any (2)   | fixpoint |        |
// -----------+-----------+-----------+----------+--------+------------------------------
// a%=b       | fixpoint  | any (2)   | fixpoint |        |
// -----------+-----------+-----------+----------+--------+------------------------------
// (double)   | fixpoint  |           | double   |        |
// -----------+-----------+-----------+----------+--------+------------------------------
// (int8_t)   | fixpoint  |           | int8_t   |        | (1)
// -----------+-----------+-----------+----------+--------+------------------------------
// (int16_t)  | fixpoint  |           | int16_t  |        | (1)
// -----------+-----------+-----------+----------+--------+------------------------------
// (int32_t)  | fixpoint  |           | int32_t  |        | (1)
// -----------+-----------+-----------+----------+--------+------------------------------
// (int64_t)  | fixpoint  |           | int64_t  |        | (1)
// -----------+-----------+-----------+----------+--------+------------------------------
// (uint8_t)  | fixpoint  |           | uint8_t  |        | (1)
// -----------+-----------+-----------+----------+--------+------------------------------
// (uint16_t) | fixpoint  |           | uint16_t |        | (1)
// -----------+-----------+-----------+----------+--------+------------------------------
// (uint32_t) | fixpoint  |           | uint32_t |        | (1)
// -----------+-----------+-----------+----------+--------+------------------------------
// (uint64_t) | fixpoint  |           | uint64_t |        | (1)
// -----------+-----------+-----------+----------+--------+------------------------------
//
// (1) In operations like a*b an intermediate result might overflow
//     if TTEMP is as narrow as TBASE.
//     Please take care of the values and ranges. If in doubt choose
//     TTEMP wider than TBASE.
//
// (2) "any" means any type which can be converted to fixpoint,
//     i.e. the same fixpoint type, any integral type, float and double.
//
//
// There is no implicit conversion between different fixpoint types.
// Please convert manually. Example:
// Anyware::fixpoint< int16_t, int32_t, 2 >   a( 4 );
// Anyware::fixpoint< int32_t, int32_t, 8 >   b( a.getBasevalue(),
//                                               1<<a.getScaleBinaryExponent()
//                                               );
//
//
////////////////////////////////////////////////////////////////////////////////
//
// There are some demo programs:
//
// - testfixpoint_linux.cpp shows basic usage and makes some tests,
//   written for a PC system running Linux. It might or might not work on
//   other systems; no GUI is needed.
//
// - testfixpoint_debug.cpp shows debug options
//
// - AVR_LCD44780_fixpoint.cpp, lcd-routines.c and lcd-routines.h are
//   used to build a test program running on an Atmel AVR (AtMega8
//   tested).
//
//   It calculates some things in a loop until 10 seconds are elapsed
//   and writes the loop count to a text LCD (Hitachi HDC44780).
//   For more information on connecting the AVR and the LCD see
//   AVR_LCD44780_fixpoint.cpp.
//   Changes on pin usage may be done in lcd-routines.h.
//
//   Inside AVR_LCD44780_fixpoint.cpp there is a typedef where you can
//   set the type used for all calculations.
//   As long as you use C types here (int16_t, uint32_t, float etc.;
//   no fixpoint) the source file may be renamed to
//   AVR_LCD44780_fixpoint.c and compiled as traditional C for
//   comparison.
//
//   There are two makefiles supplied: Makefile_AVR_c and Makefile_AVR_cpp
//   can be used to compile and flash the program on my system (atmega8,
//   16 MHz, siprog using /dev/ttyS0) the C resp. C++ version or
//   modified as needed.
//   The Makefiles are derived from the sample files at [2].
//   Caution! The original Makefile from [2] will overwrite the
//   C++ source (*.cpp) with the listing, because the replacement in the line:
//       CPPFLAGS += -Wa,-adhlns=$(<:.c=.lst)
//   fails (since the extension is not .c).
//   This leads to a listing file name identical to the source file
//   name while erasing the source file on disc.
//   I changed the listing file name to the full source name with .lst
//   appended, e.g. AVR_LCD44780_fixpoint.cpp.lst to avoid the mess.
//
////////////////////////////////////////////////////////////////////////////////
//
// reading and writing streams
//////////////////////////////
//
// TODO
//
////////////////////////////////////////////////////////////////////////////////
//
// reading and writing strings
//////////////////////////////
//
// TODO
//
////////////////////////////////////////////////////////////////////////////////
//
// Debug options
////////////////
//
//
// DEBUG_FIXPOINT
// --------------
//
// Compiling with DEBUG_FIXPOINT causes a conversion to string not
// only to write the resulting value but all internal members.
//
//
// DEBUG_FIXPOINT_CHECK_RANGE, DEBUG_FIXPOINT_CHECK_RANGE_CALLBACK
// ---------------------------------------------------------------
//
// Compiling with DEBUG_FIXPOINT_CHECK_RANGE results in more code. All
// operations will be tested whether the expected result can be
// represented. This test will be done in double in most cases.
//
// If DEBUG_FIXPOINT_CHECK_RANGE is defined then an additional
// definition of DEBUG_FIXPOINT_CHECK_RANGE_CALLBACK is
// expected to be a function (or function object).
// The function will be called on a range error.
// After return from the function the program will continue with
// incorrect values.
// DEBUG_FIXPOINT_CHECK_RANGE_CALLBACK may be defined with the name of
// a function looking like this:
//     void checkRangeMessage( double          value,
//                             const char     *pszStaticDescription,
//                             const char     *pszStaticOp1,
//                             const char     *pszStaticOp2
//                             );
// This function will be called instead of printing the message with
// these parameters:
//   value:                the value which can not be represented
//   pszStaticDescription: pointer to static buffer with description
//   pszStaticOp1:         pointer to static buffer with elements of
//                         1st operand
//   pszStaticOp2:         pointer to static buffer with elements of
//                         2nd operand, or empty string if none
// If DEBUG_FIXPOINT_CHECK_RANGE_CALLBACK is not defined a detected
// range error will cause a message printed to stderr and the program
// will continue with incorrect values.
//
// DEBUG_FIXPOINT_CHECK_RANGE requires floating point arithmetic and
// *printf functions with floating point support since the checks are
// done in double.
//
//
// DEBUG_FIXPOINT_FOLLOW_DOUBLE
// ----------------------------
//
// Compiling with DEBUG_FIXPOINT_FOLLOW_DOUBLE will cause a double
// value to be embedded in the fixpoint object.
// This object will be initialized and updated with all operations in
// floating point arithmetic along with the internal
// representation.
// An additional method double getFollowExact() returns this double
// value.
// This independent calculation allows evaluation of the results.
// <cmath> will be included internally.
//
// The debug options will obviously not speed up calculation but bloat
// the code, since the fixpoint calculations do not replace floating
// point. All computing (DEBUG_FIXPOINT_FOLLOW_DOUBLE) or some of it
// (DEBUG_FIXPOINT_CHECK_RANGE) will be done both in fixpoint and
// floating point.
// But it might be useful to work this way while feeding a program
// with typical input on a more powerful system to evaluate the
// algorithms and check the results before running the program on an
// ambedded system without DEBUG_... options.
//
//
// If any of DEBUG_FIXPOINT_CHECK_RANGE or
// DEBUG_FIXPOINT_FOLLOW_DOUBLE is defined each object may contain a
// name.
// It can be set with void setName( const char * ) and read with
// const char *getName().
// getName() may return NULL if there was no name set or no memory
// could be allocated for it in setName().
// Assigning objects will not copy the name, but initializing with
// copy-ctor will do so.
// Use this to make debug output more readable (writing to a stream
// with << prints the object name as well).
//
////////////////////////////////////////////////////////////////////////////////
//
// std::numeric_limits<>
////////////////////////
//
// std::numeric_limits<> is specialised for a each type
// Anyware::fixpoint< TBASE, TTEMP, LFRACTIONAL, ROUNDTONEAREST >.
// Thus you can retrieve the limits of any type (like min(), max(),
// digits, digits10 etc.) in the usual way.
//
// Note: min() does not give you the smallest value!
// min() gives the smallest value greater zero (as the C++ standard
// for floating point values does).
//
//
////////////////////////////////////////////////////////////////////////////////
//
// math functions
/////////////////
//
// Most functions from <cmath> are not implemeted yet.
//
// Up to now you will find only some definition for sin() and cos().
// Other functions will follow as needed.
//
//
// sin(), cos()
// ------------
//
// The default implementation of fixpoint<>sin( fixpoint<> angle )
// simply converts its argument to double and calls the std::sin()
// from the C++ lib.
//
// The default implementation of fixpoint<>cos( fixpoint<> angle )
// just calls fixpoint<>sin() with the angle moved appropriately.
//
// Since this fixpoint lib will be used to avoid internal floating
// point computing (e.g. on ambedded systems without floating point
// support) you will want to change this default behaviour.
//
// There is an additional program generate_sin_table.cpp.
// Use generate_sin_table to generate some C++ source which
// specialises the default sin() implementation and #include the
// generated C++ source in your program.
// See the comments in generate_sin_table.cpp on how to compile and
// call it.
//
// The idea is to generate source which contains a static table of sin
// values and compute the sin by interpolation. The program
// generate_sin_table expects the template parameters (TBASE, TTEMP,
// LFRACTIONAL, ROUNDTONEAREST) and a step width together with the
// name of the C++ file to be written.
//
// The step width may be adjusted internally to another value (not
// lower than the smallest differences of the angle type and rounded
// to a value 2^(-n) to speed up calculation).
// A small step width of course enlarges your program size because a
// table from 0 to pi/2 must be stored (each entry of type TBASE),
// while to large steps lead to inaccuracy.
// On the other hand small steps will not slow down calculations,
// because finding the sample points in the table is just done by
// shifting the internal base value of the angle (this is why the
// choosen step width is 2^(-n)).
//
// If you do not worry about the size of the static table choose the
// step width as 2^(-LFRACTIONAL). In this case no interpolation is
// needed at all since each internal base value (after moving it to
// [0..pi/2] directly maps to its sin value in the table - no
// multiplications needed, no divisions.
//
////////////////////////////////////////////////////////////////////////////////
//
// History:
///////////
//
// 08.07.2009 kw     - created
//                   - tested on Linux + g++ 4.3.2 and avr-g++ 4.3.2
//
// 09.07.2009 kw     - template argument TTEMP
//                   - types for intermediate values changed
//                   - more comments
//                   - test programs
//
// 08.09.2009 kw     operator--() corrected (thanks to Manuel Stahl in
//                   http://www.mikrocontroller.net/topic/143537#1404527)
//
// 27.07.2011 kw     draft of DEBUG_FIXPOINT_FOLLOW_DOUBLE, DEBUG_FIXPOINT_CHECK_RANGE
//
// 02.08.2011 kw     - numeric_limits<> (siehe [3])
//                   - checkRangeMin, checkRangeMax
//
// 03.08.2011 kw     TODOs for cmath functions
//
// 04.08.2011 kw     fixpoint(i,i) overflow corrected
//
// 06.08.2011 kw     default implementation of sin() and cos()
//
// 07.08.2011 kw     - itoa() now thread safe (no static buffer)
//                   - static double checkRangeMinLimit(),
//                     static double checkRangeMaxLimit()
//                   - name of generic cos() fixed
//
// 09.08.2011 kw     - method members in std::numeric_limits now
//                     inline
//                   - homemade replacement for stdint.h (requires
//                     -D__STDC_LIMIT_MACROS while compiling)
//
//  kw     
//
//  kw     
//
//  kw     
//
////////////////////////////////////////////////////////////////////////////////
//
// TODO:
////////
//
// - test all cases (with rounding and without)
// - external documentation
// - take care for ROUNDTONEAREST in operator ...int...
// - output of double without printf
// - implement fmod(f1,f2)
// - implement abs(f)
// - implement fabs(f)
// - implement ceil(f) kleinste int nicht kleiner als f
// - implement floor(f) grösste int nicht größer als f
// - implement sqrt(f) f>=0
// - implement pow(f1,f2) f1^f2, nicht f1==0 && f2<=0, nicht f1<=0 && f2 nicht ganzzahlig http://www.cplusplus.com/reference/clibrary/cmath/pow/
// - implement pow(f1,i2) f1^i2, nicht f1==0 && i2<=0 http://www.cplusplus.com/reference/clibrary/cmath/pow/
// - implement modf(f1,f2) http://www.cplusplus.com/reference/clibrary/cmath/modf/http://www.cplusplus.com/reference/clibrary/cmath/modf/
// - implement log10(f) http://www.cplusplus.com/reference/clibrary/cmath/log10/
// - implement log(f) http://www.cplusplus.com/reference/clibrary/cmath/log/ http://www.mikrocontroller.net/topic/84017
// - implement ldexp(f1,i2) http://www.cplusplus.com/reference/clibrary/cmath/ldexp/
// - implement frexp(f1,*i2) evtl.? http://www.cplusplus.com/reference/clibrary/cmath/frexp/
// - implement exp(f) http://www.cplusplus.com/reference/clibrary/cmath/exp/
// - implement tanh(f) http://www.cplusplus.com/reference/clibrary/cmath/tanh/
// - implement sinh (f) http://www.cplusplus.com/reference/clibrary/cmath/sinh/
// - implement atan2(f1,f2) http://www.cplusplus.com/reference/clibrary/cmath/atan2/
// - implement atan(f) http://www.cplusplus.com/reference/clibrary/cmath/atan/
// - implement asin(f) http://www.cplusplus.com/reference/clibrary/cmath/asin/
// - implement acos(f) http://www.cplusplus.com/reference/clibrary/cmath/acos/
// - implement tan(f) http://www.cplusplus.com/reference/clibrary/cmath/tan/
// - implement sin(f) http://www.cplusplus.com/reference/clibrary/cmath/sin/
// - implement cos(f) http://www.cplusplus.com/reference/clibrary/cmath/cos/
// - conversion to string
// - output to ostream (modifier to change format? 1.25 | 5/4 | (basevalue=5 scale=4))
// - read from istream
// - build format specifier for printf
// - maybe wrapper for C (#ifndef __cplusplus...
// - 
// - 
// - 
// - 
// - 
//
////////////////////////////////////////////////////////////////////////////////
//
// [1] Wikipedia Fixed-point arithmetic
//     http://en.wikipedia.org/wiki/Fixed-point_arithmetic
//
// [2] http://www.mikrocontroller.net/
//
// [3] http://stdcxx.apache.org/doc/stdlibref/numeric-limits.html
//
// [4] http://www.wachtler.de/#fixpoint
//
// [5] http://www.mikrocontroller.net/topic/143537
//
// [6] http://www.mikrocontroller.net/topic/227823
//
//
////////////////////////////////////////////////////////////////////////////////


#ifndef _FIXPOINT_H_
#define _FIXPOINT_H_

// only valid in C++, not C
#ifdef __cplusplus


#include <stdint.h>
//#include <inttypes.h>
#include <cmath>


// define _FIXPOINT_VIRTUAL if and only if derivation of fixpoint is
// to be expected. Then some methods will become virtual.
#ifdef _FIXPOINT_VIRTUAL
#define __FIXPOINT_VIRTUAL  virtual
#else
#define __FIXPOINT_VIRTUAL
#endif /* ifdef _FIXPOINT_VIRTUAL ... else ... */


// A prior definition of NO_STDSTRING suppresses the conversion to
// std::string and eliminates the need of #include <string>
//
// A prior definition of NO_STDIOSTREAM suppresses output to
// std::ostream and input from std::istream.

#ifndef NO_STDSTRING
#include <string>
#endif /* ifndef NO_STDSTRING */

#ifndef NO_STDIOSTREAM
#include <iostream>
#endif /* ifndef NO_STDIOSTREAM */

#ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE
#include <cmath>
#endif /* ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE */

#ifdef DEBUG_FIXPOINT_CHECK_RANGE
#include <cstdio>
#endif /* ifdef DEBUG_FIXPOINT_CHECK_RANGE */


#if defined DEBUG_FIXPOINT_FOLLOW_DOUBLE || defined DEBUG_FIXPOINT_CHECK_RANGE
#include <cstdlib>
#include <cstring>
#endif /* if defined DEBUG_FIXPOINT_FOLLOW_DOUBLE || defined DEBUG_FIXPOINT_CHECK_RANGE */

#include <limits>


namespace Anyware
{

  template< typename TBASE = int16_t,
            typename TTEMP = TBASE,
            const int LFRACTIONAL = sizeof(TBASE)*4,
            const bool ROUNDTONEAREST = true
            > class fixpoint
  {

    public:

    // this type
    typedef    fixpoint< TBASE, TTEMP, LFRACTIONAL, ROUNDTONEAREST >   type;

    //
    // constructing
    //

    // ctor with no arguments: initializes to 0
    fixpoint()
      : basevalue( 0 )
#ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE
      , followExact( 0.0 )
#endif /* ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE */
#if defined DEBUG_FIXPOINT_FOLLOW_DOUBLE || defined DEBUG_FIXPOINT_CHECK_RANGE
      , objectName( NULL )
#endif /* if defined DEBUG_FIXPOINT_FOLLOW_DOUBLE || defined DEBUG_FIXPOINT_CHECK_RANGE */
    {
#ifdef DEBUG_FIXPOINT_CHECK_RANGE
      checkRange( 0.0, "ctor", __FILE__, __LINE__, __FUNCTION__, NULL );
#endif /* ifdef DEBUG_FIXPOINT_CHECK_RANGE */
    }

    // ctor with integral value
    fixpoint( TBASE init )
      : basevalue( LFRACTIONAL>0
                   ? ( init << LFRACTIONAL )
                   : ( ROUNDTONEAREST
                       ? ( (((TTEMP)init)+(1<<(-LFRACTIONAL-1))) >> -LFRACTIONAL )
                       : ( init >> -LFRACTIONAL )
                       )
                   )
#ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE
      , followExact( double(init) )
#endif /* ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE */
#if defined DEBUG_FIXPOINT_FOLLOW_DOUBLE || defined DEBUG_FIXPOINT_CHECK_RANGE
      , objectName( NULL )
#endif /* if defined DEBUG_FIXPOINT_FOLLOW_DOUBLE || defined DEBUG_FIXPOINT_CHECK_RANGE */
    {
#ifdef DEBUG_FIXPOINT_CHECK_RANGE
      checkRange( double(init), "ctor with integral value", __FILE__, __LINE__, __FUNCTION__, NULL );
#endif /* ifdef DEBUG_FIXPOINT_CHECK_RANGE */
    }

    // ctor with floating point value
    fixpoint( const double &init )
      : basevalue( ldexp( init, LFRACTIONAL )
                   +
                   ( ROUNDTONEAREST ? ( init<0.0 ? -0.5 : +0.5 ) : 0.0 )
                   )
#ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE
      , followExact( init )
#endif /* ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE */
#if defined DEBUG_FIXPOINT_FOLLOW_DOUBLE || defined DEBUG_FIXPOINT_CHECK_RANGE
      , objectName( NULL )
#endif /* if defined DEBUG_FIXPOINT_FOLLOW_DOUBLE || defined DEBUG_FIXPOINT_CHECK_RANGE */
    {
#ifdef DEBUG_FIXPOINT_CHECK_RANGE
      checkRange( init, "ctor with double", __FILE__, __LINE__, __FUNCTION__, NULL );
#endif /* ifdef DEBUG_FIXPOINT_CHECK_RANGE */
    }

    // ctor with a and b: initial value is a/b
    template< typename TA, typename TB > fixpoint( TA a, TB b )

      : basevalue( 0 )
#ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE
      , followExact( double(a)/double(b) )
#endif /* ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE */
#if defined DEBUG_FIXPOINT_FOLLOW_DOUBLE || defined DEBUG_FIXPOINT_CHECK_RANGE
      , objectName( NULL )
#endif /* if defined DEBUG_FIXPOINT_FOLLOW_DOUBLE || defined DEBUG_FIXPOINT_CHECK_RANGE */
    {

      if( b<0 )
      {
        a=-a;
        b=-b;
      }
      // basevalue must be corected with the scaling factor.
      // This may be done before dividing seeing a possible overflow,
      // or after dividing seeing a potential underrun loosing digits.
      if( sizeof(TTEMP)>sizeof(TBASE) )
      {
        // the first argument can be shifted to the left without loss
        // by using TTEMP
        basevalue = ( ( ( LFRACTIONAL>0
                          ? ( ROUNDTONEAREST
                              ? ( a>=0
                                  ? (((((TTEMP)a)<<LFRACTIONAL)+(b/2))/b)
                                  : (((((TTEMP)a)<<LFRACTIONAL)-(b/2))/b)
                                  )
                              : (((TTEMP)a)<<LFRACTIONAL)/b
                              )
                          : ( ROUNDTONEAREST
                              ? ( a>0
                                  ? ((a+(((TTEMP)b)<<(-LFRACTIONAL-1)))
                                     /(((TTEMP)b)<<-LFRACTIONAL))
                                  : ((a-b/2)/(((TTEMP)b)<<-LFRACTIONAL))
                                  )
                              : (a/(((TTEMP)b)<<-LFRACTIONAL)) )
                          )
                        )
                      );
      }
      else
      {
        // shifting of the first argument may loose leading digits!
        // So we shift as far as possible before dividing, the rest
        // afterwards
        int    toBeShifted;
        if( LFRACTIONAL>0 )
        {
          toBeShifted = LFRACTIONAL;
          TBASE  aShiftTest;
          // TODO: perhaps take a faster way than shift one by one in
          // a loop
          for( size_t i=0; i<LFRACTIONAL; ++i )
          {
            aShiftTest = a<<1;
            if( aShiftTest>>1 == a )
            {
              // shift possible
              a <<= 1;
              toBeShifted--;
            }
            else
            {
              break;
            }
          }
        }
        else
        {
          toBeShifted = -LFRACTIONAL;
          TBASE  bShiftTest;
          for( size_t i=0; i<-LFRACTIONAL; ++i )
          {
            bShiftTest = b<<1;
            if( bShiftTest>>1 == b )
            {
              // shift possible
              b <<= 1;
              toBeShifted--;
            }
            else
            {
              break;
            }
          }
        }
        basevalue = ( ROUNDTONEAREST
                      ? ( a>=0
                          ? ( ( a + (b/2) )/b )
                          : ( ( a - (b/2) )/b )
                          )
                      : a/b
                      );
        basevalue <<= toBeShifted;
      }
#ifdef DEBUG_FIXPOINT_CHECK_RANGE
      checkRange( double(a)/double(b), "ctor with a/b", __FILE__, __LINE__, __FUNCTION__, NULL );
#endif /* ifdef DEBUG_FIXPOINT_CHECK_RANGE */
    }

    // copy-ctor
    fixpoint( const fixpoint &init )
      : basevalue( init.basevalue )
#ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE
      , followExact( init.followExact )
#endif /* ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE */
#if defined DEBUG_FIXPOINT_FOLLOW_DOUBLE || defined DEBUG_FIXPOINT_CHECK_RANGE
      , objectName( NULL )
#endif /* if defined DEBUG_FIXPOINT_FOLLOW_DOUBLE || defined DEBUG_FIXPOINT_CHECK_RANGE */
    {
#if defined DEBUG_FIXPOINT_FOLLOW_DOUBLE || defined DEBUG_FIXPOINT_CHECK_RANGE
      setName( init.getName() );
#endif /* if defined DEBUG_FIXPOINT_FOLLOW_DOUBLE || defined DEBUG_FIXPOINT_CHECK_RANGE */
    }


    __FIXPOINT_VIRTUAL ~fixpoint()
    {
#if defined DEBUG_FIXPOINT_FOLLOW_DOUBLE || defined DEBUG_FIXPOINT_CHECK_RANGE
      std::free( objectName ); objectName = NULL;
#endif /* if defined DEBUG_FIXPOINT_FOLLOW_DOUBLE || defined DEBUG_FIXPOINT_CHECK_RANGE */
    }

    //
    // conversions
    //

    operator double() const
    {
      return ldexp( double( basevalue ), int(-LFRACTIONAL) );
    }

    // TODO: take care for ROUNDTONEAREST
#define _FIXPOINT_DEFINE_CONVERSION_TO_INTEGRAL(integraltype)   \
    operator integraltype() const                               \
    {                                                           \
      if( LFRACTIONAL>0 )                                       \
      {                                                         \
        return basevalue >> LFRACTIONAL;                        \
      }                                                         \
      else if( LFRACTIONAL<0 )                                  \
      {                                                         \
        return ((integraltype)basevalue) << -LFRACTIONAL;       \
      }                                                         \
      else                                                      \
      {                                                         \
        return basevalue;                                       \
      }                                                         \
    }

    _FIXPOINT_DEFINE_CONVERSION_TO_INTEGRAL(int8_t);
    _FIXPOINT_DEFINE_CONVERSION_TO_INTEGRAL(int16_t);
    _FIXPOINT_DEFINE_CONVERSION_TO_INTEGRAL(int32_t);
    _FIXPOINT_DEFINE_CONVERSION_TO_INTEGRAL(int64_t);
    _FIXPOINT_DEFINE_CONVERSION_TO_INTEGRAL(uint8_t);
    _FIXPOINT_DEFINE_CONVERSION_TO_INTEGRAL(uint16_t);
    _FIXPOINT_DEFINE_CONVERSION_TO_INTEGRAL(uint32_t);
    _FIXPOINT_DEFINE_CONVERSION_TO_INTEGRAL(uint64_t);

#undef _FIXPOINT_DEFINE_CONVERSION_TO_INTEGRAL

    operator bool() const
    {
      return basevalue!=0;
    }

    //
    // operations
    //

    // !a
    bool operator!() const
    {
      return basevalue==0;
    }

    // prefix++: ++a returns modified value
    type &operator++()
    {
#ifdef DEBUG_FIXPOINT_CHECK_RANGE
      checkRange( std::ldexp( double(basevalue), int(-LFRACTIONAL) )+1.0, "pre increment", __FILE__, __LINE__, __FUNCTION__, NULL );
#endif /* ifdef DEBUG_FIXPOINT_CHECK_RANGE */
      if( LFRACTIONAL>=0 )
      {
        basevalue += 1<<LFRACTIONAL;
      }
#ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE
      followExact++;
#endif /* ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE */
      return *this;
    }

    // postfix++: a++ returns unmodified value
    type operator++( int )
    {
#ifdef DEBUG_FIXPOINT_CHECK_RANGE
      checkRange( std::ldexp( double(basevalue), int(-LFRACTIONAL) )+1.0, "post increment", __FILE__, __LINE__, __FUNCTION__, NULL );
#endif /* ifdef DEBUG_FIXPOINT_CHECK_RANGE */
      type oldvalue( *this );
      ++*this;
      return oldvalue;
    }

    // prefix--: --a returns modified value
    type &operator--()
    {
#ifdef DEBUG_FIXPOINT_CHECK_RANGE
      checkRange( std::ldexp( double(basevalue), int(-LFRACTIONAL) )-1.0, "pre decrement", __FILE__, __LINE__, __FUNCTION__, NULL );
#endif /* ifdef DEBUG_FIXPOINT_CHECK_RANGE */
      if( LFRACTIONAL>=0 )
      {
        basevalue -= 1<<LFRACTIONAL;
      }
#ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE
      followExact--;
#endif /* ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE */
      return *this;
    }

    // postfix--: a-- returns unmodified value
    type operator--( int )
    {
#ifdef DEBUG_FIXPOINT_CHECK_RANGE
      checkRange( std::ldexp( double(basevalue), int(-LFRACTIONAL) )-1.0, "post decrement", __FILE__, __LINE__, __FUNCTION__, NULL );
#endif /* ifdef DEBUG_FIXPOINT_CHECK_RANGE */
      type oldvalue( *this );
      --*this;
      return oldvalue;
    }

    // +a does nothing
    type operator+() const
    {
      return *this;
    }

    // -a negates
    type operator-() const
    {
#ifdef DEBUG_FIXPOINT_CHECK_RANGE
      checkRange( std::ldexp( -double(basevalue), int(-LFRACTIONAL) ), "negatiation", __FILE__, __LINE__, __FUNCTION__, NULL );
#endif /* ifdef DEBUG_FIXPOINT_CHECK_RANGE */
      type ret;
      ret.basevalue = -basevalue;
#ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE
      ret.followExact = -followExact;
#endif /* ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE */
      return ret;
    }

    type operator*( const type &rS ) const
    {
#ifdef DEBUG_FIXPOINT_CHECK_RANGE
      checkRange( ( std::ldexp( double(basevalue), int(-LFRACTIONAL) )
                    *std::ldexp( double(rS.basevalue), int(-LFRACTIONAL) )
                    ),
                  "multiplication",
                  __FILE__, __LINE__, __FUNCTION__, &rS
                  );
#endif /* ifdef DEBUG_FIXPOINT_CHECK_RANGE */
      TTEMP  tempvalue = TTEMP(basevalue) * rS.basevalue;
      if( LFRACTIONAL>0 )
      {
        if( ROUNDTONEAREST )
        {
          // get a bit away from 0 to have the result rounded instead
          // of truncating
          if( tempvalue>0 )
          {
            tempvalue += 1<<(LFRACTIONAL-1);
          }
          else if( tempvalue<0 )
          {
            tempvalue -= 1<<(LFRACTIONAL-1);
          }
        }
        tempvalue >>= LFRACTIONAL;
      }
      else if( LFRACTIONAL<0 )
      {
        tempvalue <<= -LFRACTIONAL;
      }
      type ret;
      ret.basevalue = tempvalue;
#ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE
      ret.followExact = followExact*rS.followExact;
#endif /* ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE */
      return ret;
    }

    type operator/( const type &rS ) const
    {
#ifdef DEBUG_FIXPOINT_CHECK_RANGE
      checkRange( ( std::ldexp( double(basevalue), int(-LFRACTIONAL) )
                    /std::ldexp( double(rS.basevalue), int(-LFRACTIONAL) )
                    ),
                  "division",
                  __FILE__, __LINE__, __FUNCTION__, &rS
                  );
#endif /* ifdef DEBUG_FIXPOINT_CHECK_RANGE */
      if( LFRACTIONAL>0 )
      {
        type   ret( TTEMP(basevalue),
                    TTEMP(rS.basevalue)
                    );
#ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE
        ret.followExact = followExact/rS.followExact;
#endif /* ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE */
        return ret;
      }
      else
      {
        type  ret( basevalue, rS.basevalue );
#ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE
        ret.followExact = followExact/rS.followExact;
#endif /* ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE */
        return ret;
      }
    }

    type operator%( const type &rS ) const
    {
#ifdef DEBUG_FIXPOINT_CHECK_RANGE
      checkRange( fmod( std::ldexp( double(basevalue), int(-LFRACTIONAL) ),
                        std::ldexp( double(rS.basevalue), int(-LFRACTIONAL) )
                        ),
                  "modulo",
                  __FILE__, __LINE__, __FUNCTION__, &rS
                  );
#endif /* ifdef DEBUG_FIXPOINT_CHECK_RANGE */
      type ret;
      ret.basevalue = basevalue % rS.basevalue;
#ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE
      ret.followExact = std::fmod( followExact, rS.followExact );
#endif /* ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE */
      return ret;
    }

    // a+b
    type operator+( const type &rS ) const
    {
#ifdef DEBUG_FIXPOINT_CHECK_RANGE
      checkRange( ( std::ldexp( double(basevalue), int(-LFRACTIONAL) )
                    +std::ldexp( double(rS.basevalue), int(-LFRACTIONAL) )
                    ),
                  "addition",
                  __FILE__, __LINE__, __FUNCTION__, &rS
                  );
#endif /* ifdef DEBUG_FIXPOINT_CHECK_RANGE */
      type ret;
      ret.basevalue = basevalue + rS.basevalue;
#ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE
        ret.followExact = followExact + rS.followExact;
#endif /* ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE */
      return ret;
    }

    // a-b
    type operator-( const type &rS ) const
    {
#ifdef DEBUG_FIXPOINT_CHECK_RANGE
      checkRange( ( std::ldexp( double(basevalue), int(-LFRACTIONAL) )
                    -std::ldexp( double(rS.basevalue), int(-LFRACTIONAL) )
                    ),
                  "subtraction",
                  __FILE__, __LINE__, __FUNCTION__, &rS
                  );
#endif /* ifdef DEBUG_FIXPOINT_CHECK_RANGE */
      type ret;
      ret.basevalue = basevalue - rS.basevalue;
#ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE
        ret.followExact = followExact - rS.followExact;
#endif /* ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE */
      return ret;
    }

    // a<b
    bool operator<( const type &rS ) const
    {
      return basevalue<rS.basevalue;
    }

    // a>b
    bool operator>( const type &rS ) const
    {
      return basevalue>rS.basevalue;
    }

    // a<=b
    bool operator<=( const type &rS ) const
    {
      return basevalue<=rS.basevalue;
    }

    // a>=b
    bool operator>=( const type &rS ) const
    {
      return basevalue>=rS.basevalue;
    }

    // a==b
    bool operator==( const type &rS ) const
    {
      return basevalue==rS.basevalue;
    }

    // a!=b
    bool operator!=( const type &rS ) const
    {
      return basevalue!=rS.basevalue;
    }

    // a&&b
    bool operator&&( const type &rS ) const
    {
      return basevalue&&rS.basevalue;
    }

    // a||b
    bool operator||( const type &rS ) const
    {
      return basevalue||rS.basevalue;
    }

    // a=b
    const type &operator=( const type &rS )
    {
      basevalue = type( rS ).basevalue;
#ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE
      followExact = rS.followExact;
#endif /* ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE */
      return *this;
    }

    // a+=b
    const type &operator+=( const type &rS )
    {
#ifdef DEBUG_FIXPOINT_CHECK_RANGE
      checkRange( ( std::ldexp( double(basevalue), int(-LFRACTIONAL) )
                    +std::ldexp( double(rS.basevalue), int(-LFRACTIONAL) )
                    ),
                  "addition and assignment",
                  __FILE__, __LINE__, __FUNCTION__, &rS
                  );
#endif /* ifdef DEBUG_FIXPOINT_CHECK_RANGE */
      basevalue = ( *this + type( rS ) ).basevalue;
#ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE
      followExact += rS.followExact;
#endif /* ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE */
      return *this;
    }

    // a-=b
    const type &operator-=( const type &rS )
    {
#ifdef DEBUG_FIXPOINT_CHECK_RANGE
      checkRange( ( std::ldexp( double(basevalue), int(-LFRACTIONAL) )
                    -std::ldexp( double(rS.basevalue), int(-LFRACTIONAL) )
                    ),
                  "subtraction and assignment",
                  __FILE__, __LINE__, __FUNCTION__, &rS
                  );
#endif /* ifdef DEBUG_FIXPOINT_CHECK_RANGE */
      basevalue = ( *this - type( rS ) ).basevalue;
#ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE
      followExact -= rS.followExact;
#endif /* ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE */
      return *this;
    }

    // a*=b
    const type &operator*=( const type &rS )
    {
#ifdef DEBUG_FIXPOINT_CHECK_RANGE
      checkRange( ( std::ldexp( double(basevalue), int(-LFRACTIONAL) )
                    *std::ldexp( double(rS.basevalue), int(-LFRACTIONAL) )
                    ),
                  "multiplication and assignment",
                  __FILE__, __LINE__, __FUNCTION__, &rS
                  );
#endif /* ifdef DEBUG_FIXPOINT_CHECK_RANGE */
      basevalue = ( *this * type( rS ) ).basevalue;
#ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE
      followExact *= rS.followExact;
#endif /* ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE */
      return *this;
    }

    // a/=b
    const type &operator/=( const type &rS )
    {
#ifdef DEBUG_FIXPOINT_CHECK_RANGE
      checkRange( ( std::ldexp( double(basevalue), int(-LFRACTIONAL) )
                    /std::ldexp( double(rS.basevalue), int(-LFRACTIONAL) )
                    ),
                  "division and assignment",
                  __FILE__, __LINE__, __FUNCTION__, &rS
                  );
#endif /* ifdef DEBUG_FIXPOINT_CHECK_RANGE */
      basevalue = ( *this / type( rS ) ).basevalue;
#ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE
      followExact /= rS.followExact;
#endif /* ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE */
      return *this;
    }

    // a%=b
    const type &operator%=( const type &rS )
    {
#ifdef DEBUG_FIXPOINT_CHECK_RANGE
      checkRange( std::fmod( std::ldexp( double(basevalue), int(-LFRACTIONAL) ),
                             std::ldexp( double(rS.basevalue), int(-LFRACTIONAL) )
                        ),
                  "modulo and assignment",
                  __FILE__, __LINE__, __FUNCTION__, &rS
                  );
#endif /* ifdef DEBUG_FIXPOINT_CHECK_RANGE */
      basevalue = ( *this % type( rS ) ).basevalue;
#ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE
      followExact = std::fmod( followExact, rS.followExact );
#endif /* ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE */
      return *this;
    }




    // setss the raw internal value
    //
    // The represented value might be calculated with
    // basevalue/(2^scalebinaryexponent) or basevalue/(scale)
    void setBasevalue( TBASE newBasevalue )
    {
      basevalue = newBasevalue;
    }

    // returns the raw internal value
    //
    // The represented value might be calculated with
    // basevalue/(2^scalebinaryexponent) or basevalue/(scale)
    TBASE getBasevalue() const
    {
      return basevalue;
    }

    // returns the binary exponent of the scale factor (length of
    // fractional part)
    //
    // The represented value might be calculated with
    // basevalue/(2^scalebinaryexponent)
    int getScaleBinaryExponent() const
    {
      return LFRACTIONAL;
    }

    // returns the scale factor
    //
    // The represented value might be calculated with
    // basevalue/(scale)
    double getScale() const
    {
      if( LFRACTIONAL>0 )
      {
        return double(1<<LFRACTIONAL);
      }
      else if( LFRACTIONAL<0 )
      {
        return 1.0/double(1<<-LFRACTIONAL);
      }
      else
      {
        return 1.0;
      }
    }

#ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE
    double getFollowExact()
    {
      return followExact;
    }
#endif /* ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE */

#if defined DEBUG_FIXPOINT_FOLLOW_DOUBLE || defined DEBUG_FIXPOINT_CHECK_RANGE
    const char *getName() const
    {
      return objectName;
    }

    void setName( const char *newName )
    {
      std::free( objectName );
      if( newName )
      {
        objectName = (char *)std::malloc( std::strlen( newName )+1 );
        if( objectName )
        {
          std::strcpy( objectName, newName );
        }
      }
    }
#endif /* if defined DEBUG_FIXPOINT_FOLLOW_DOUBLE || defined DEBUG_FIXPOINT_CHECK_RANGE */


#ifndef NO_STDIOSTREAM

    // Value will be converted to double for writing into stream.
    // Thus all floating point formatting from include <iomanip> like
    // std::setprecision(4) will work.
    __FIXPOINT_VIRTUAL void printToOstream( std::ostream &os ) const
    {
      char bufferBasevalue[66];
#ifdef DEBUG_FIXPOINT
      // test version: write internal value, scaling and resulting
      // value:
      os << "fixpoint( internal = " << itoa( bufferBasevalue, sizeof(bufferBasevalue), basevalue, 10 )
         << ", scaling=" << LFRACTIONAL
         << ", rounding=" << ROUNDTONEAREST
         << ", resulting=" << double( *this )
#ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE
         << ", followExact=" << followExact
#endif /* ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE */
#if defined DEBUG_FIXPOINT_FOLLOW_DOUBLE || defined DEBUG_FIXPOINT_CHECK_RANGE
         << ", objectName=\"" << ( objectName ? objectName : "?" ) << "\""
#endif /* if defined DEBUG_FIXPOINT_FOLLOW_DOUBLE || defined DEBUG_FIXPOINT_CHECK_RANGE */
         << " )";
#else
      // final version: write resulting value:
      os << double( *this );
#endif /* ifdef DEBUG_FIXPOINT */
    }


    __FIXPOINT_VIRTUAL void readFromIstream( std::istream &is )
    {
      double   dValue;
      is >> dValue;
      *this = dValue;
#ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE
      followExact = dValue;
#endif /* ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE */

    }

#endif /* ifndef NO_STDIOSTREAM */


  private:

    TBASE        basevalue;

    // converts an integral (up to 64 Bit) to a zero terminated
    // string describing value in given base (max 16).
    // If TBASE is a signed type then the output will respect the
    // sign.
    //
    // Return value is the start of the zero terminated string which
    // may differ from the start address of the given buffer since the
    // string will be set up at the end of buffer and written to lower
    // addresses as required.
    //
    // No checks will be made.
    //
    // The required buffer length depends on the value and the base
    // and will reach 65 for a (u)int64_t with base==2 (including
    // terminating '\0').
    //
    // The function is thread safe as long as the given buffer is not
    // global storage.
    static const char *itoa( char *buffer, size_t lBuffer, TBASE value, int base )
    {
      char         *p = buffer+lBuffer; // start from back end
      *--p = '\0';                      // terminate

      bool isNegative = false;
      if( std::numeric_limits<TBASE>::min()<0 && value<0 )
      {
        isNegative = true;
      }

      if( value )
      {
        while( value )
        {
          TBASE     remainder = value%base;
          if( std::numeric_limits<TBASE>::min()<0 && value<0 )
          {
            value = -value;
            remainder = -remainder;
          }
          value /= base;
          *--p = "0123456789ABCDEF"[remainder];
        }
        if( isNegative )
        {
          *--p = '-';
        }
      }
      else
      {
        // zero value
        *--p = '0';
      }

      return p;
    }

    //static const char *itoa( TBASE value, int base )
    //{
    //  static char   puffer[66];
    //  char         *p = puffer+sizeof(puffer); // start from back end
    //  *--p = '\0';                             // terminate
    //
    //  bool isNegative = false;
    //  if( std::numeric_limits<TBASE>::min()<0 && value<0 )
    //  {
    //    isNegative = true;
    //  }
    //
    //  if( value )
    //  {
    //    while( value )
    //    {
    //      TBASE     remainder = value%base;
    //      if( std::numeric_limits<TBASE>::min()<0 && value<0 )
    //      {
    //        value = -value;
    //        remainder = -remainder;
    //      }
    //      value /= base;
    //      *--p = "0123456789ABCDEF"[remainder];
    //    }
    //    if( isNegative )
    //    {
    //      *--p = '-';
    //    }
    //  }
    //  else
    //  {
    //    // zero value
    //    *--p = '0';
    //  }
    //
    //  return p;
    //}


#ifdef DEBUG_FIXPOINT_CHECK_RANGE

    // writes *this as string to pResult (up to nMax characters)
    const char *fprint( char *pResult, size_t nMax ) const
    {
      char   bufferBasevalue[66];
      char   bufferLFractional[66];
#ifdef _WIN32
      _snprintf
#else
      std::snprintf
#endif /* ifdef _WIN32 */
        ( pResult,
          nMax,
          "fixpoint( internal=%s"
          ", scaling=%s"
          ", rounding=%s"
          ", resulting=%g"
#ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE
          ", followExact=%g"
#endif /* ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE */
#if defined DEBUG_FIXPOINT_FOLLOW_DOUBLE || defined DEBUG_FIXPOINT_CHECK_RANGE
          ", objectName=%s"
#endif /* if defined DEBUG_FIXPOINT_FOLLOW_DOUBLE || defined DEBUG_FIXPOINT_CHECK_RANGE */
          " )",
          itoa( bufferBasevalue, sizeof(bufferBasevalue), basevalue, 10 ),
          itoa( bufferLFractional, sizeof(bufferLFractional), LFRACTIONAL, 10 ),
          ( ROUNDTONEAREST ? "to nearest" : "truncate" ),
          double( *this )
#ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE
          , followExact
#endif /* ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE */
#if defined DEBUG_FIXPOINT_FOLLOW_DOUBLE || defined DEBUG_FIXPOINT_CHECK_RANGE
          , objectName
#endif /* if defined DEBUG_FIXPOINT_FOLLOW_DOUBLE || defined DEBUG_FIXPOINT_CHECK_RANGE */
          );
      return pResult;
    }

    static void checkRangeMessage( double          value,
                                   const char     *pszStaticDescription,
                                   const char     *pszStaticOp1,
                                   const char     *pszStaticOp2
                                   )
    {
      fprintf( stderr,
               "%s\nop: %s %s%s",
               pszStaticDescription,
               pszStaticOp1,
               ( pszStaticOp2 && *pszStaticOp2 ? " op2: " : "" ),
               ( pszStaticOp2 && *pszStaticOp2 ? pszStaticOp2 : "" )
               );
    }

  public:

#ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE
    double       followExact;
#endif /* ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE */

    // checks given value against range limits
    void checkRange( double          value,
                     const           char *description,
                     const           char *filename,
                     int             line,
                     const char     *fun,
                     const fixpoint *p_2ndOp
                     ) const
    {
      // update overall min and max values:
      checkRangeMin = std::min( checkRangeMin, value );
      checkRangeMax = std::max( checkRangeMax, value );

      static const double valueMin = std::ldexp( double(std::numeric_limits<TBASE>::min()), int(-LFRACTIONAL) );
      static const double valueMax = std::ldexp( double(std::numeric_limits<TBASE>::max()), int(-LFRACTIONAL) );

      if( value<valueMin || value>valueMax )
      {
        static char description[100];
        static char op1[100];
        static char op2[100];
#ifdef _WIN32
        _snprintf
#else
        std::snprintf
#endif /* ifdef _WIN32 */
          ( description,
            sizeof(description),
            "range error fixpoint<>: value %g not in [%g,%g]"
            " in %s : %d in %s (%s)",
            value, valueMin, valueMax,
            filename, line, fun,
            description
            );
#ifdef DEBUG_FIXPOINT_CHECK_RANGE_CALLBACK
        DEBUG_FIXPOINT_CHECK_RANGE_CALLBACK
#else
        checkRangeMessage
#endif /* ifdef DEBUG_FIXPOINT_CHECK_RANGE_CALLBACK */
          ( value,
            description,
            this->fprint( op1, sizeof(op1) ),
            (p_2ndOp ? this->fprint( op2, sizeof(op2) ) : "" )
            );

      }
    }

    static double   checkRangeMin;
    static double   checkRangeMax;

    static double checkRangeMinLimit()
    {
      return std::ldexp( std::numeric_limits< TBASE >::min(), int( -LFRACTIONAL ) );
    }

    static double checkRangeMaxLimit()
    {
      return std::ldexp( std::numeric_limits< TBASE >::max(), int( -LFRACTIONAL ) );
    }

#endif /* ifdef DEBUG_FIXPOINT_CHECK_RANGE */



#if defined DEBUG_FIXPOINT_FOLLOW_DOUBLE || defined DEBUG_FIXPOINT_CHECK_RANGE
    char        *objectName;
#endif /* defined DEBUG_FIXPOINT_FOLLOW_DOUBLE || defined DEBUG_FIXPOINT_CHECK_RANGE */

    friend class std::numeric_limits< Anyware::fixpoint< TBASE, TTEMP, LFRACTIONAL, ROUNDTONEAREST > >;

  }; // template class fixpoint


  // (non optimal) default implementation of sin(), may be specialized
  // for each template instantiation e.g. using interpolation in
  // tables instead of std::sin()
  template < typename TBASE,
             typename TTEMP,
             const int LFRACTIONAL,
             const bool ROUNDTONEAREST
             >
  inline
  fixpoint< TBASE, TTEMP, LFRACTIONAL, ROUNDTONEAREST >
  sin( const fixpoint< TBASE, TTEMP, LFRACTIONAL, ROUNDTONEAREST > &angle )
  {
    Anyware::fixpoint< TBASE, TTEMP, LFRACTIONAL, ROUNDTONEAREST > ret ( ::sin( double(angle) ) );
#ifdef DEBUG_FIXPOINT_CHECK_RANGE
    ret.checkRange( std::sin( double(angle) ),
                    "sin() default implementation",
                    __FILE__, __LINE__, __FUNCTION__, NULL
                    );
#endif /* ifdef DEBUG_FIXPOINT_CHECK_RANGE */

#ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE
    ret.followExact = std::sin( angle.followExact );
#endif /* ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE */

    return ret;
  }

  // default implementation of cos()
  template < typename TBASE,
             typename TTEMP,
             const int LFRACTIONAL,
             const bool ROUNDTONEAREST
             >
  inline
  fixpoint< TBASE, TTEMP, LFRACTIONAL, ROUNDTONEAREST >
  cos( const fixpoint< TBASE, TTEMP, LFRACTIONAL, ROUNDTONEAREST > &angle )
  {
    // The return value is not calculated by std::cos(double)
    // directly. Instead the sin(fixpoint) version is used.
    // This way cos(fixpoint) can benefit from a fast specialisation of
    // fixpoint::sin() without any effort.
    Anyware::fixpoint< TBASE, TTEMP, LFRACTIONAL, ROUNDTONEAREST >
      ret ( sin( angle + Anyware::fixpoint< TBASE, TTEMP, LFRACTIONAL, ROUNDTONEAREST >(M_PI/2.0) ) );
#ifdef DEBUG_FIXPOINT_CHECK_RANGE
    ret.checkRange( ::sin( double(angle) ),
                    "sin() default implementation",
                    __FILE__, __LINE__, __FUNCTION__, NULL
                    );
#endif /* ifdef DEBUG_FIXPOINT_CHECK_RANGE */

#ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE
    ret.followExact = std::cos( angle.followExact );
#endif /* ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE */

    return ret;
  }

#ifndef NO_STDIOSTREAM

template< typename TBASE,
          typename TTEMP,
          const int LFRACTIONAL,
          const bool ROUNDTONEAREST
          >
inline std::ostream &operator<<
( std::ostream &os,
  const Anyware::fixpoint< TBASE, TTEMP, LFRACTIONAL, ROUNDTONEAREST > &f
  )
{
  f.printToOstream( os );
  return os;
}

template< typename TBASE,
          typename TTEMP,
          const int LFRACTIONAL,
          const bool ROUNDTONEAREST
          >
inline std::istream &operator>>
( std::istream &is,
  const Anyware::fixpoint< TBASE, TTEMP, LFRACTIONAL, ROUNDTONEAREST > &f
  )
{
  f.readFromIstream( is );
  return is;
}

#endif /* ifndef NO_STDIOSTREAM */


} // namespace Anyware


/////////////////////////////////////////////////////////////////////////////

// specialize std::numeric_limits
// See [3]

namespace std
{

  template< typename TBASE, typename TTEMP, const int LFRACTIONAL, const bool ROUNDTONEAREST >
    class numeric_limits< Anyware::fixpoint< TBASE, TTEMP, LFRACTIONAL, ROUNDTONEAREST > >
  {
  public:

    inline static Anyware::fixpoint< TBASE, TTEMP, LFRACTIONAL, ROUNDTONEAREST > denorm_min() throw()
    {
      Anyware::fixpoint< TBASE, TTEMP, LFRACTIONAL, ROUNDTONEAREST > ret;
      ret.basevalue = 1;
#ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE
      ret.followExact = double(ret);
#endif /* ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE */
      return ret;
    }

    static const int digits = std::numeric_limits< TBASE >::digits;

    static const int digits10 = std::numeric_limits< TBASE >::digits10;

    inline static Anyware::fixpoint< TBASE, TTEMP, LFRACTIONAL, ROUNDTONEAREST > epsilon() throw()
    {
      Anyware::fixpoint< TBASE, TTEMP, LFRACTIONAL, ROUNDTONEAREST > ret;
      ret.basevalue = 1;
#ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE
      ret.followExact = double(ret);
#endif /* ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE */
      return ret;
    }

    static const std::float_denorm_style has_denorm = std::denorm_absent;

    static const bool has_denorm_loss = false;

    static const bool has_infinity = false;

    static const bool has_quiet_NaN = false;

    static const bool has_signaling_NaN = false;

    inline static Anyware::fixpoint< TBASE, TTEMP, LFRACTIONAL, ROUNDTONEAREST > infinity() throw()
    {
      Anyware::fixpoint< TBASE, TTEMP, LFRACTIONAL, ROUNDTONEAREST > ret;
      ret.basevalue = std::numeric_limits< TBASE >::max();
#ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE
      ret.followExact = double(ret);
#endif /* ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE */
      return ret;
    }

    static const bool is_bounded = true;

    static const bool is_exact = true;

    static const bool is_iec559 = false;

    static const bool is_integer = false;

    static const bool is_modulo = true;

    static const bool is_signed = std::numeric_limits< TBASE >::is_signed;

    static const bool is_specialized = true;

    inline static Anyware::fixpoint< TBASE, TTEMP, LFRACTIONAL, ROUNDTONEAREST > max() throw()
    {
      Anyware::fixpoint< TBASE, TTEMP, LFRACTIONAL, ROUNDTONEAREST > ret;
      ret.basevalue = std::numeric_limits< TBASE >::max();
#ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE
      ret.followExact = double(ret);
#endif /* ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE */
      return ret;
    }

    static const int max_exponent = 0;

    static const int max_exponent10 = 0;

    inline static Anyware::fixpoint< TBASE, TTEMP, LFRACTIONAL, ROUNDTONEAREST > min() throw()
    {
      Anyware::fixpoint< TBASE, TTEMP, LFRACTIONAL, ROUNDTONEAREST > ret;
      ret.basevalue = 1;
#ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE
      ret.followExact = double(ret);
#endif /* ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE */
      return ret;
    }

    static const int min_exponent = 0;

    static const int min_exponent10 = 0;

    inline static Anyware::fixpoint< TBASE, TTEMP, LFRACTIONAL, ROUNDTONEAREST > quiet_NaN() throw()
    {
      Anyware::fixpoint< TBASE, TTEMP, LFRACTIONAL, ROUNDTONEAREST > ret;
      ret.basevalue = 0;
#ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE
      ret.followExact = double(ret);
#endif /* ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE */
      return ret;
    }

    static const int radix = std::numeric_limits< TBASE >::radix;

    inline static Anyware::fixpoint< TBASE, TTEMP, LFRACTIONAL, ROUNDTONEAREST > round_error() throw()
    {
      Anyware::fixpoint< TBASE, TTEMP, LFRACTIONAL, ROUNDTONEAREST > ret;
      ret.basevalue = 1;
#ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE
      ret.followExact = double(ret);
#endif /* ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE */
      return ret;
    }

    static const float_round_style round_style = round_toward_zero;

    inline static Anyware::fixpoint< TBASE, TTEMP, LFRACTIONAL, ROUNDTONEAREST > signaling_NaN() throw()
    {
      Anyware::fixpoint< TBASE, TTEMP, LFRACTIONAL, ROUNDTONEAREST > ret;
      ret.basevalue = 0;
#ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE
      ret.followExact = double(ret);
#endif /* ifdef DEBUG_FIXPOINT_FOLLOW_DOUBLE */
      return ret;
    }

    static const bool tinyness_before = false;

    static const bool traps = false;

  }; // class numeric_limits< Anyware::fixpoint< TBASE, TTEMP, LFRACTIONAL, ROUNDTONEAREST > >
} // namespace std


#endif /* ifdef __cplusplus */

#endif /* ifndef _FIXPOINT_H_ */

// Local Variables:
// mode:C++
// End:
