// Time-stamp: "09.07.09 11:11 lcd-routines.c klaus?wachtler.de"
//
// Ansteuerung eines HD44780 kompatiblen LCD im 4-Bit-Interfacemodus
// [1] http://www.mikrocontroller.net/articles/AVR-GCC-Tutorial
// [2] http://www.mikrocontroller.net/topic/88543#751982
//
// Die Pinbelegung ist ber defines in lcd-routines.h einstellbar
//
// nderungen:
// 23.06.2009 kw    - von auen nicht aufzurufende Funktionen static
//                    gemacht:
//                    lcd_data(), lcd_command(), lcd_enable()
//                  - gemeinsamen Code aus lcd_command() und lcd_data()
//                    in neue Funktion lcd_byte() ausgelagert
//                  - Shiften und Maskieren in lcd_byte() eliminiert
//                  - in lcd_set_cursor() Adressberechnung korrigiert
//                    fr blaues LCD "Blue Line" von Electronic Assembly
//                    EA W204B-NLW bzw. Reichelt LCD204BBL


// wird bei mir im Makefile gemacht!
#ifndef F_CPU
#error F_CPU nicht gesetzt
#endif


#include <avr/io.h>
#include "lcd-routines.h"
#include <util/delay.h>


// LCD Befehle

#define LCD_CLEAR_DISPLAY 0x01
#define LCD_CURSOR_HOME   0x02



// erzeugt den Enable-Puls
static void lcd_enable( void )
{
   // Bei Problemen ggf. Pause gem Datenblatt des LCD Controllers einfgen

   // http://www.mikrocontroller.net/topic/81974#685882
   LCD_EN1_PORT |= (1<<LCD_EN1);

    _delay_us(1);                   // kurze Pause
   // Bei Problemen ggf. Pause gem Datenblatt des LCD Controllers verlngern
   // http://www.mikrocontroller.net/topic/80900
   LCD_EN1_PORT &= ~(1<<LCD_EN1);
}


// sendet ein Daten- oder Kommandobyte (RS am LCD mu vorher
// entsprechend gesetzt sein)
static void lcd_byte( unsigned char data )
{
  // oberes Nibble auf den LCD-Pins 4..7 ausgeben:
  LCD_PORT_4 &= ~(1<<LCD_D4);
  LCD_PORT_5 &= ~(1<<LCD_D5);
  LCD_PORT_6 &= ~(1<<LCD_D6);
  LCD_PORT_7 &= ~(1<<LCD_D7);

  if( data & 0x10 ) LCD_PORT_4 |= (1<<LCD_D4);// setzen
  if( data & 0x20 ) LCD_PORT_5 |= (1<<LCD_D5);
  if( data & 0x40 ) LCD_PORT_6 |= (1<<LCD_D6);
  if( data & 0x80 ) LCD_PORT_7 |= (1<<LCD_D7);
  lcd_enable();

  // unteres Nibble auf den LCD-Pins 4..7 ausgeben:
  LCD_PORT_4 &= ~(1<<LCD_D4);
  LCD_PORT_5 &= ~(1<<LCD_D5);
  LCD_PORT_6 &= ~(1<<LCD_D6);
  LCD_PORT_7 &= ~(1<<LCD_D7);

  if( data & 0x01 ) LCD_PORT_4 |= (1<<LCD_D4);// setzen
  if( data & 0x02 ) LCD_PORT_5 |= (1<<LCD_D5);
  if( data & 0x04 ) LCD_PORT_6 |= (1<<LCD_D6);
  if( data & 0x08 ) LCD_PORT_7 |= (1<<LCD_D7);
  lcd_enable();

  // Dem LCD Zeit geben, das Kommando auszufhren
  _delay_us(42);
}


// sendet ein Datenbyte an das LCD
static void lcd_data( unsigned char data )
{
   LCD_RS_PORT |= (1<<LCD_RS);        // RS auf 1 setzen
   lcd_byte( data );
}


// sendet einen Befehl an das LCD
static void lcd_command( unsigned char command )
{
   LCD_RS_PORT &= ~(1<<LCD_RS);        // RS auf 0 setzen
   lcd_byte( command );
}


// Initialisierung:
// Muss ganz am Anfang des Programms aufgerufen werden.
void lcd_init( void )
{
   // Ports auf Ausgang schalten
   LCD_DDR_4 |=(1<<LCD_D4);
   LCD_DDR_5 |=(1<<LCD_D5);
   LCD_DDR_6 |=(1<<LCD_D6);
   LCD_DDR_7 |=(1<<LCD_D7);

   LCD_EN1_DDR |= (1<<LCD_EN1);

   LCD_RS_DDR  |= (1<<LCD_RS);

   // muss 3 mal hintereinander gesendet werden zur Initialisierung

   _delay_ms(15);

   LCD_PORT_4 |=  (1<<LCD_D4);
   LCD_PORT_5 |=  (1<<LCD_D5);
   LCD_PORT_6 &= ~(1<<LCD_D6);
   LCD_PORT_7 &= ~(1<<LCD_D7);

   LCD_RS_PORT &= ~(1<<LCD_RS);      // RS auf 0
   lcd_enable();

   _delay_ms(5);
   lcd_enable();

   _delay_ms(1);
   lcd_enable();
   _delay_ms(1);

   // 4 Bit Modus aktivieren
   LCD_PORT_4 &=  ~(1<<LCD_D4);
   LCD_PORT_5 |=   (1<<LCD_D5);
   LCD_PORT_6 &=  ~(1<<LCD_D6);
   LCD_PORT_7 &=  ~(1<<LCD_D7);

   lcd_enable();
   _delay_ms(1);

   // 4Bit / 2 Zeilen / 5x7
   lcd_command(0x28);

   // Display ein / Cursor aus / kein Blinken
   lcd_command(0x0C);

   // inkrement / kein Scrollen
   lcd_command(0x06);

   lcd_clear();
}


// Sendet den Befehl zur Lschung des Displays
void lcd_clear( void )
{
   lcd_command(LCD_CLEAR_DISPLAY);
   _delay_ms(5);
}


// Sendet den Befehl: Cursor Home
void lcd_home( void )
{
   lcd_command(LCD_CURSOR_HOME);
   _delay_ms(5);
}


// setzt den Cursor in Zeile y (1..4) Spalte x (0..15)
void lcd_set_cursor( uint8_t x, uint8_t y )
{
  uint8_t tmp;

  switch (y)
  {
    // Originalversion von [1]:
    //case 1: tmp=0x80+0x00+x;  break;    // 1. Zeile
    //case 2: tmp=0x80+0x40+x;  break;    // 2. Zeile
    //case 3: tmp=0x80+0x10+x;  break;    // 3. Zeile
    //case 4: tmp=0x80+0x50+x;  break;    // 4. Zeile

    // Klappt mit 4x20 blau ("Blue Line" von Electronic Assembly
    // EA W204B-NLW bzw. Reichelt LCD204BBL)
    case 1: tmp=0x80+0x00+x;  break;    // 1. Zeile
    case 2: tmp=0x80+0x40+x;  break;    // 2. Zeile
    case 3: tmp=0x80+0x14+x;  break;    // 3. Zeile
    case 4: tmp=0x80+0x54+x;  break;    // 4. Zeile
    default: return;
  }
  lcd_command(tmp);
}


// Schreibt einen String auf das LCD
void lcd_string( const char *data )
{
    while(*data)
    {
      lcd_data(*data);
      data++;
    }
}
